package org.tk.sqle.model;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.jdom.Element;
import org.tk.sqle.Application;
import org.jdom.JDOMException;

/** Texte. 
 * Hilfetexte aus helpcon.rtf, helpstmt.rtf und helpresult.rtf<br>
 * SQL Befehle und Schlsselwrter aus sql.xml<br>
 * Typen:<br>
 * 1. Einleitung des Befehls. Davon hngt auch die Zuordnung selectSQL oder updateSQL<br>
 * 2. Innerhalb der Befehle verwendete Schlsselwrter. Sie gelten meist fr mehrere Befehle.<br>
 * 3. sql-Datentypen<br>
 * @author Thiemo Kahn
 */
public final class Help {

	private static Element rootE = null;
	public static final int HELPCON = 0;
	public static final int HELPSTMT = 1;
	public static final int HELPRESULT = 2;

	/** Liest SQL-Schlsselwrter und Befehle.
	 */
	private static void readSQLKeyWords() {
		XML xml;
		try {
			xml = new XML(Application.getRoot() + "/text/sql.xml");
			rootE = xml.getRootElement();
		} catch (JDOMException e) {
			Log.error("Text.readSQLKeyWords: ", e);
		}
	}

	/** Liefert Hilfetext fr Verbindungs-, Statement-, Ergebnisbereich.
	 * @param type HELPCON, HELPSTMT, HELPRESULT
	 * @return Hilfetext
	 */
	public static String getHelpText(int type) {
		String s = "";
		FileReader fr = null;
		BufferedReader br = null;
		try {
			String path = Application.getRoot() + "/text/helpcon.rtf";
			if (type == HELPSTMT)
				path = Application.getRoot() + "/text/helpstmt.rtf";
			else if (type == HELPRESULT)
				path = Application.getRoot() + "/text/helpresult.rtf";
			fr = new FileReader(path);
			br = new BufferedReader(fr);
			String line = null;
			while ((line = br.readLine()) != null) {
				s = s + line + Application.lineSep;
			}

		} catch (FileNotFoundException e) {
			Log.error("Text.getHelpText: ", e);
		} catch (IOException e) {
			Log.error("Text.getHelpText: ", e);
		} finally {
			if (br != null)
				try {
					br.close();
				} catch (IOException e1) {
					Log.error("Help.getHelpText: ", e1);
				}
			if (fr != null)
				try {
					fr.close();
				} catch (IOException e2) {
					Log.error("Help.getHelpText: ", e2);
				}
		}

		return s;
	}

	/** Liefert Schlsselwort anhand eines bergebenen Wortanfangs.<br>
	 * Das Schlsselwort wird in Grobuchstaben zurckgegeben.<br>
	 * @param begin Wortanfang
	 * @return Liste mit Schlsselwrtern in Grobuchstaben als Strings.
	 */
	static Vector getSQLKeyWords(String begin) {
		if (rootE == null)
			readSQLKeyWords();
		Vector keys = new Vector();
		List l = rootE.getChildren();
		for (Iterator iter = l.iterator(); iter.hasNext();) {
			Element el = (Element) iter.next();
			if (el.getChildText("name") != null
				&& el.getChildText("name").startsWith(begin.toUpperCase())) {
				keys.add(el.getChildText("name"));
			}
		}
		if (keys.size() == 0)
			keys = null;
		return keys;
	}

	/** Liefert Beschreibung zu einem Schlsselwort.
	 * @param keyWord Schlsselwort.
	 * @return Beschreibung
	 */
	public static String getDescriptionForSQLKeyWord(String keyWord) {
		String s = null;
		if (rootE == null)
			readSQLKeyWords();
		List l = rootE.getChildren("result");
		for (Iterator iter = l.iterator(); iter.hasNext();) {
			Element el = (Element) iter.next();
			if (el.getChildText("name").equalsIgnoreCase(keyWord)) {
				s = el.getChildText("desc");
				break;
			}
		}
		if (s == null) {
			l = rootE.getChildren("cmd");
			for (Iterator iter = l.iterator(); iter.hasNext();) {
				Element el = (Element) iter.next();
				if (el.getChildText("name").equalsIgnoreCase(keyWord)) {
					s = el.getChildText("desc");
					break;
				}
			}
		}
		if (s == null) {
			l = rootE.getChildren("word");
			for (Iterator iter = l.iterator(); iter.hasNext();) {
				Element el = (Element) iter.next();
				if (el.getChildText("name").equalsIgnoreCase(keyWord)) {
					s = el.getChildText("desc");
					break;
				}
			}
		}
		if (s != null) {
			s = s.replaceAll("\t", "");
		}
		return s;
	}

	/** Prft, ob das bergebene Statement einen Befehl mit Rckgabe einleitet.<br>
	 * Beispiel: SELECT.<br>
	 * @return true = Befehl mit Ergebnis, false = kein Befehl oder Befehl ohne Rckgabe
	 * @param stmt SQL-Befehl
	 */
	static boolean isSQLCommandWithResult(String stmt) {
		boolean b = false;
		if (rootE == null)
			readSQLKeyWords();
		List l = rootE.getChildren("result");
		for (Iterator iter = l.iterator(); iter.hasNext();) {
			String[] sa = ((Element) iter.next()).getChildTextTrim("name").split(" ");
			String n = sa[0].toUpperCase();
			if (stmt.toUpperCase().startsWith(n)) {
				b = true;
				break;
			}
		}
		return b;
	}

	/** Prft, ob das bergebene Statement einen Befehl ohne Rckgabe einleitet.<br>
	 * Beispiel: INSERT.<br>
	 * @param stmt SQL-Befehl
	 * @return true = Befehl ohne Ergebnis, false = kein Befehl oder Befehl mit Rckgabe
	 */
	static boolean isSQLCommandWithoutResult(String stmt) {
		boolean b = false;
		if (rootE == null)
			readSQLKeyWords();
		List l = rootE.getChildren("cmd");
		for (Iterator iter = l.iterator(); iter.hasNext();) {
			String[] sa = ((Element) iter.next()).getChildTextTrim("name").split(" ");
			String n = sa[0].toUpperCase();
			if (stmt.toUpperCase().startsWith(n)) {
				b = true;
				break;
			}
		}
		return b;
	}

}
